# Sample "stack-pop" Script for pxWorks
# Author: cloudcell
# Description: 
#   1) adds data to internal state (and consumes, i.e. deletes it afterwards)
#   2) if activated by a "signal," pops one item from the top of the stack


# Starting Up ------------------------------------------------------------------
# set working directory to the directory of the script:
# setwd(dirname(parent.frame(2)$ofile))
message(getwd()) # shows current working directory
# stop()

# the environment creates this file ("paths.meta")
paths_to_get_input_data <- read.delim(file="paths.meta", header = F, as.is = T)
# ------------------------------------------------------------------------------

# define a global variable
state_file <- "state"

# step 1: if there's an INIT signal, load the data and pop the first item and save state
sig_fp <- paths_to_get_input_data[2,]
if(file.exists(sig_fp)) {
    message("init:")

    # step 1.1: add data to internal state
    # check if data exists
    data_fp <- paths_to_get_input_data[1,]
    
    if(file.exists(data_fp)){
        if(!file.exists(state_file)) {
            message("state file does not exist: creating state file")
            file.create(state_file)
        }
        message("appending data to state file")
        
        file.append(state_file,data_fp)
    }


    # try reading state file
    # state <- read.csv(state_file, header = F)
    
    tmp <- try(read.csv(state_file, header = F, as.is = T))
    if (inherits(tmp, 'try-error')) {
        message("nothing to read in state file, returning empty data frame")
        state <- data.frame(x= character(0))
    }  else {
        state <- tmp
    }
    
    if(nrow(state)>0){
        message("popping one item from the state table")
        
        write(state$V1[1],file="tmp.pop")
        file.rename("tmp.pop","pop")

        write(state$V1[-1],file="tmp.state")
        file.rename("tmp.state",state_file)
    } else {
        # return nothing
    }
    file.remove(sig_fp)
} 
    
    
    # step 2: if there's a signal, pop the first item and save state
    sig_fp <- paths_to_get_input_data[3,]
    if(file.exists(sig_fp)) {
        
        # make sure the new item is requested, and not just a refeed
        msg <- read.csv(sig_fp, header = T, as.is = T)
        if(msg$x=="NEXT") {
            
            message("new item requested, popping the data from the stack")
            
            # try reading state file
            tmp <- try(read.csv(state_file, header = F, as.is = T))
            if (inherits(tmp, 'try-error')) {
                message("state table is not readable, returning empty data.frame")
                state <- data.frame(x= character(0))
            } else {
                state <- tmp
            }
            
            
            if(nrow(state)>0){
                message("returning the data from the stack")
                
                write(state$V1[1],file="tmp.pop")
                file.rename("tmp.pop","pop")
                
                write(state$V1[-1],file="tmp.state")
                file.rename("tmp.state",state_file)
            } else {
                # return nothing
            }
            
        }
        file.remove(sig_fp)
    }
    


# Saving Work ------------------------------------------------------------------
# The write must be an atomic operation to prevent conflicts over file
# when the file is being written to / flushed from memory to drive
# write.csv(output,"tmp.output")
# file.rename(from="tmp.output",to="output") # atomic operation
message("Done.")
# ------------------------------------------------------------------------------
